@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright © 2021 Connectomics Lab
+# Copyright © 2021-2022 Connectomics Lab
 # University Hospital Center and University of Lausanne (UNIL-CHUV), Switzerland,
 # and contributors
 #
@@ -11,6 +11,7 @@
 # General imports
 import os
 import sys
+import datetime
 
 # Configuration of the graphical backend of traitsui
 # Note: Should be at the very beginning before any
@@ -19,6 +20,9 @@ from traits.etsconfig.api import ETSConfig
 ETSConfig.toolkit = 'qt'  # noqa: E402
 os.environ['QT_API'] = 'pyqt5'  # noqa: E402
 
+# Suppress QXcbConnection: XCB error
+os.environ['QT_LOGGING_RULES'] = '*.debug=false;qt.qpa.*=false'  # noqa: E402
+
 from bids import BIDSLayout
 
 # Own imports
@@ -49,16 +53,16 @@ def main():
     #####################
 
     # 1. Check if the BIDS directory exists
-    if args.bids_dir and not os.path.exists(args.bids_dir):
+    if args.dataset_dir and not os.path.exists(args.dataset_dir):
         print(
-            f"The provided BIDS directory ({args.bids_dir}) does not exists"
+            f"The provided input dataset directory ({args.dataset_dir}) does not exists"
         )
         exit_code = 1
         return exit_code
-    elif args.bids_dir and os.path.exists(args.bids_dir):
+    elif args.dataset_dir and os.path.exists(args.dataset_dir) and not args.is_not_bids:
         # 2. Check if the BIDS dataset is successfully loaded by pybids
         try:
-            layout = BIDSLayout(args.bids_dir)
+            layout = BIDSLayout(args.dataset_dir)
             print(f'PyBIDS summary of input dataset:\n{layout}')
         except Exception as e:
             print(f'{e}')
@@ -97,6 +101,7 @@ def main():
 
     # Commandline mode
     if not args.gui:
+
         # Handle the type of sibling for annexing data
         if args.git_annex_ssh_special_sibling_config:
             git_annex_special_sibling_config = args.git_annex_ssh_special_sibling_config
@@ -106,25 +111,31 @@ def main():
             sibling_type = 'osf'
         # Create a NeuroDataPubProject
         neurodatapub_project = NeuroDataPubProject(
-            bids_dir=args.bids_dir,
+            dataset_dir=args.dataset_dir,
+            dataset_is_bids=not args.is_not_bids,
             datalad_dataset_dir=args.datalad_dir,
             git_annex_special_sibling_config=git_annex_special_sibling_config,
             sibling_type=sibling_type,
             github_sibling_config=args.github_sibling_config,
-            mode=args.mode
+            mode=args.mode,
+            generate_script=args.generate_script
         )
         print(neurodatapub_project)
 
+        # Initialize the script that will log all commands generated
+        cmd_log = '#!/bin/sh\n\n'
+
         if args.mode == "create-only" or args.mode == "all":
             print(
                 "\n############################################\n"
                 "# Creation of Datalad Dataset\n"
                 "############################################\n"
             )
-            res = neurodatapub_project.create_datalad_dataset()
+            res, cmd_fun_log = neurodatapub_project.create_datalad_dataset()
             if res:
                 exit_code = 0
                 print('Success')
+                cmd_log += f'{cmd_fun_log}\n'
             else:
                 exit_code = 1
                 print('An error occurred during the creation of the Datalad dataset')
@@ -135,7 +146,7 @@ def main():
                 "# Configuration of the publication siblings\n"
                 "############################################\n"
             )
-            res = neurodatapub_project.configure_siblings()
+            res, cmd_fun_log = neurodatapub_project.configure_siblings()
             if not res:
                 exit_code = 1
                 print('An error occurred during the configuration of the publication siblings')
@@ -145,13 +156,31 @@ def main():
                 "# Publication of Datalad Dataset\n"
                 "############################################\n"
             )
-            res = neurodatapub_project.publish_datalad_dataset()
+            cmd_log += f'{cmd_fun_log}\n'
+            res, cmd_fun_log = neurodatapub_project.publish_datalad_dataset()
             if res:
                 exit_code = 0
                 print('Success')
+                cmd_log += f'{cmd_fun_log}\n'
             else:
                 exit_code = 1
                 print('An error occurred during the publication of the Datalad dataset')
+        if args.generate_script:
+            # Create name of script with time stamp
+            now = datetime.datetime.now()
+            script_basename = f'neurodatapub_{now.strftime("%d-%m-%Y_%H-%M-%S")}.sh'
+            # Create the code folder if it does not exist
+            if not os.path.exists(os.path.join(args.dataset_dir, 'code')):
+                os.makedirs(os.path.join(args.dataset_dir, 'code'), exist_ok=True)
+            script_path = os.path.join(args.dataset_dir, 'code', script_basename)
+            print(
+                "\n############################################\n"
+                f"# Generation of script {script_path}\n"
+                "############################################\n"
+            )
+            with open(script_path, 'w') as f:
+                f.writelines(cmd_log)
+
     else:
         # GUI mode
         print(
@@ -171,12 +200,14 @@ def main():
             sibling_type = 'ssh'
         # Create a NeuroDataPubProjectUI
         neurodatapub_project_gui = NeuroDataPubProjectUI(
-                bids_dir=args.bids_dir,
+                dataset_dir=args.dataset_dir,
+                dataset_is_bids=not args.is_not_bids,
                 datalad_dataset_dir=args.datalad_dir,
                 git_annex_special_sibling_config=git_annex_special_sibling_config,
                 sibling_type=sibling_type,
                 github_sibling_config=args.github_sibling_config,
-                mode=args.mode
+                mode=args.mode,
+                generate_script=args.generate_script
         )
         print(neurodatapub_project_gui)
 
